import { DecodeStream } from './decode_stream.js';

class AsciiHexStream extends DecodeStream {
	constructor(str, maybeLength) {
		if (maybeLength) {
			maybeLength *= 0.5;
		}

		super(maybeLength);
		this.str = str;
		this.dict = str.dict;
		this.firstDigit = -1;
	}

	readBlock() {
		const UPSTREAM_BLOCK_SIZE = 8000;
		const bytes = this.str.getBytes(UPSTREAM_BLOCK_SIZE);

		if (!bytes.length) {
			this.eof = true;
			return;
		}

		const maxDecodeLength = bytes.length + 1 >> 1;
		const buffer = this.ensureBuffer(this.bufferLength + maxDecodeLength);
		let bufferLength = this.bufferLength;
		let firstDigit = this.firstDigit;

		for (const ch of bytes) {
			let digit;

			if (ch >= 0x30 && ch <= 0x39) {
				digit = ch & 0x0f;
			} else if (ch >= 0x41 && ch <= 0x46 || ch >= 0x61 && ch <= 0x66) {
				digit = (ch & 0x0f) + 9;
			} else if (ch === 0x3e) {
				this.eof = true;
				break;
			} else {
				continue;
			}

			if (firstDigit < 0) {
				firstDigit = digit;
			} else {
				buffer[bufferLength++] = firstDigit << 4 | digit;
				firstDigit = -1;
			}
		}

		if (firstDigit >= 0 && this.eof) {
			buffer[bufferLength++] = firstDigit << 4;
			firstDigit = -1;
		}

		this.firstDigit = firstDigit;
		this.bufferLength = bufferLength;
	}
}

export { AsciiHexStream };
