import { unreachable, shadow, bytesToString } from '../shared/util.js';

class BaseStream {
	constructor() {
		if (this.constructor === BaseStream) {
			unreachable("Cannot initialize BaseStream.");
		}
	}

	get length() {
		unreachable("Abstract getter `length` accessed");
	}

	get isEmpty() {
		unreachable("Abstract getter `isEmpty` accessed");
	}

	get isDataLoaded() {
		return shadow(this, "isDataLoaded", true);
	}

	getByte() {
		unreachable("Abstract method `getByte` called");
	}

	getBytes(length) {
		unreachable("Abstract method `getBytes` called");
	}

	peekByte() {
		const peekedByte = this.getByte();

		if (peekedByte !== -1) {
			this.pos--;
		}

		return peekedByte;
	}

	peekBytes(length) {
		const bytes = this.getBytes(length);
		this.pos -= bytes.length;
		return bytes;
	}

	getUint16() {
		const b0 = this.getByte();
		const b1 = this.getByte();

		if (b0 === -1 || b1 === -1) {
			return -1;
		}

		return (b0 << 8) + b1;
	}

	getInt32() {
		const b0 = this.getByte();
		const b1 = this.getByte();
		const b2 = this.getByte();
		const b3 = this.getByte();
		return (b0 << 24) + (b1 << 16) + (b2 << 8) + b3;
	}

	getByteRange(begin, end) {
		unreachable("Abstract method `getByteRange` called");
	}

	getString(length) {
		return bytesToString(this.getBytes(length));
	}

	skip(n) {
		this.pos += n || 1;
	}

	reset() {
		unreachable("Abstract method `reset` called");
	}

	moveStart() {
		unreachable("Abstract method `moveStart` called");
	}

	makeSubStream(start, length, dict = null) {
		unreachable("Abstract method `makeSubStream` called");
	}

	getBaseStreams() {
		return null;
	}
}

export { BaseStream };
