import { stringToUTF8String, warn } from '../shared/util.js';
import { parseXFAPath } from './core_utils.js';
import { SimpleXMLParser } from './xml_parser.js';

function decodeString(str) {
	try {
		return stringToUTF8String(str);
	} catch(ex) {
		warn(`UTF-8 decoding failed: "${ex}".`);
		return str;
	}
}

class DatasetXMLParser extends SimpleXMLParser {
	constructor(options) {
		super(options);
		this.node = null;
	}

	onEndElement(name) {
		const node = super.onEndElement(name);

		if (node && name === "xfa:datasets") {
			this.node = node;
			throw new Error("Aborting DatasetXMLParser.");
		}
	}
}

class DatasetReader {
	constructor(data) {
		if (data.datasets) {
			this.node = new SimpleXMLParser({ hasAttributes: true }).parseFromString(data.datasets).documentElement;
		} else {
			const parser = new DatasetXMLParser({ hasAttributes: true });

			try {
				parser.parseFromString(data["xdp:xdp"]);
			} catch(_) {
				
			}

			this.node = parser.node;
		}
	}

	getValue(path) {
		if (!this.node || !path) {
			return "";
		}

		const node = this.node.searchNode(parseXFAPath(path), 0);

		if (!node) {
			return "";
		}

		const first = node.firstChild;

		if (first && first.nodeName === "value") {
			return node.children.map(child => decodeString(child.textContent));
		}

		return decodeString(node.textContent);
	}
}

export { DatasetReader };
