import { DecodeStream } from './decode_stream.js';

const chunkSize = 512;

class DecryptStream extends DecodeStream {
	constructor(str, maybeLength, decrypt) {
		super(maybeLength);
		this.str = str;
		this.dict = str.dict;
		this.decrypt = decrypt;
		this.nextChunk = null;
		this.initialized = false;
	}

	readBlock() {
		let chunk;

		if (this.initialized) {
			chunk = this.nextChunk;
		} else {
			chunk = this.str.getBytes(chunkSize);
			this.initialized = true;
		}

		if (!chunk || chunk.length === 0) {
			this.eof = true;
			return;
		}

		this.nextChunk = this.str.getBytes(chunkSize);
		const hasMoreData = this.nextChunk && this.nextChunk.length > 0;
		const decrypt = this.decrypt;
		chunk = decrypt(chunk, !hasMoreData);

		const bufferLength = this.bufferLength,
			newLength = bufferLength + chunk.length,
			buffer = this.ensureBuffer(newLength);

		buffer.set(chunk, bufferLength);
		this.bufferLength = newLength;
	}
}

export { DecryptStream };
