import { stringToPDFString, warn } from '../shared/util.js';
import { BaseStream } from './base_stream.js';
import { Dict } from './primitives.js';

function pickPlatformItem(dict) {
	if (dict.has("UF")) {
		return dict.get("UF");
	} else if (dict.has("F")) {
		return dict.get("F");
	} else if (dict.has("Unix")) {
		return dict.get("Unix");
	} else if (dict.has("Mac")) {
		return dict.get("Mac");
	} else if (dict.has("DOS")) {
		return dict.get("DOS");
	}

	return null;
}

class FileSpec {
	constructor(root, xref) {
		if (!(root instanceof Dict)) {
			return;
		}

		this.xref = xref;
		this.root = root;

		if (root.has("FS")) {
			this.fs = root.get("FS");
		}

		this.description = root.has("Desc")
		? stringToPDFString(root.get("Desc"))
		: "";

		if (root.has("RF")) {
			warn("Related file specifications are not supported");
		}

		this.contentAvailable = true;

		if (!root.has("EF")) {
			this.contentAvailable = false;
			warn("Non-embedded file specifications are not supported");
		}
	}

	get filename() {
		if (!this._filename && this.root) {
			const filename = pickPlatformItem(this.root) || "unnamed";
			this._filename = stringToPDFString(filename).replaceAll("\\\\", "\\").replaceAll("\\/", "/").replaceAll("\\", "/");
		}

		return this._filename;
	}

	get content() {
		if (!this.contentAvailable) {
			return null;
		}

		if (!this.contentRef && this.root) {
			this.contentRef = pickPlatformItem(this.root.get("EF"));
		}

		let content = null;

		if (this.contentRef) {
			const fileObj = this.xref.fetchIfRef(this.contentRef);

			if (fileObj instanceof BaseStream) {
				content = fileObj.getBytes();
			} else {
				warn("Embedded file specification points to non-existing/invalid content");
			}
		} else {
			warn("Embedded file specification does not have a content");
		}

		return content;
	}

	get serializable() {
		return {
			filename: this.filename,
			content: this.content
		};
	}
}

export { FileSpec };
