import { BaseStream } from './base_stream.js';
import { DecodeStream } from './decode_stream.js';
import { Dict } from './primitives.js';
import { Jbig2Image } from './jbig2.js';
import { shadow } from '../shared/util.js';

class Jbig2Stream extends DecodeStream {
	constructor(stream, maybeLength, params) {
		super(maybeLength);
		this.stream = stream;
		this.dict = stream.dict;
		this.maybeLength = maybeLength;
		this.params = params;
	}

	get bytes() {
		return shadow(this, "bytes", this.stream.getBytes(this.maybeLength));
	}

	ensureBuffer(requested) {
		
	}

	readBlock() {
		if (this.eof) {
			return;
		}

		const jbig2Image = new Jbig2Image();
		const chunks = [];

		if (this.params instanceof Dict) {
			const globalsStream = this.params.get("JBIG2Globals");

			if (globalsStream instanceof BaseStream) {
				const globals = globalsStream.getBytes();

				chunks.push({
					data: globals,
					start: 0,
					end: globals.length
				});
			}
		}

		chunks.push({
			data: this.bytes,
			start: 0,
			end: this.bytes.length
		});

		const data = jbig2Image.parseChunks(chunks);
		const dataLength = data.length;

		for (let i = 0; i < dataLength; i++) {
			data[i] ^= 0xff;
		}

		this.buffer = data;
		this.bufferLength = dataLength;
		this.eof = true;
	}
}

export { Jbig2Stream };
