import { DecodeStream } from './decode_stream.js';
import { JpxImage } from './jpx.js';
import { shadow } from '../shared/util.js';

class JpxStream extends DecodeStream {
	constructor(stream, maybeLength, params) {
		super(maybeLength);
		this.stream = stream;
		this.dict = stream.dict;
		this.maybeLength = maybeLength;
		this.params = params;
	}

	get bytes() {
		return shadow(this, "bytes", this.stream.getBytes(this.maybeLength));
	}

	ensureBuffer(requested) {
		
	}

	readBlock() {
		if (this.eof) {
			return;
		}

		const jpxImage = new JpxImage();
		jpxImage.parse(this.bytes);
		const width = jpxImage.width;
		const height = jpxImage.height;
		const componentsCount = jpxImage.componentsCount;
		const tileCount = jpxImage.tiles.length;

		if (tileCount === 1) {
			this.buffer = jpxImage.tiles[0].items;
		} else {
			const data = new Uint8ClampedArray(width * height * componentsCount);

			for (let k = 0; k < tileCount; k++) {
				const tileComponents = jpxImage.tiles[k];
				const tileWidth = tileComponents.width;
				const tileHeight = tileComponents.height;
				const tileLeft = tileComponents.left;
				const tileTop = tileComponents.top;
				const src = tileComponents.items;
				let srcPosition = 0;
				let dataPosition = (width * tileTop + tileLeft) * componentsCount;
				const imgRowSize = width * componentsCount;
				const tileRowSize = tileWidth * componentsCount;

				for (let j = 0; j < tileHeight; j++) {
					const rowBytes = src.subarray(srcPosition, srcPosition + tileRowSize);
					data.set(rowBytes, dataPosition);
					srcPosition += tileRowSize;
					dataPosition += imgRowSize;
				}
			}

			this.buffer = data;
		}

		this.bufferLength = this.buffer.length;
		this.eof = true;
	}
}

export { JpxStream };
