import { DecodeStream } from './decode_stream.js';

class RunLengthStream extends DecodeStream {
	constructor(str, maybeLength) {
		super(maybeLength);
		this.str = str;
		this.dict = str.dict;
	}

	readBlock() {
		const repeatHeader = this.str.getBytes(2);

		if (!repeatHeader || repeatHeader.length < 2 || repeatHeader[0] === 128) {
			this.eof = true;
			return;
		}

		let buffer;
		let bufferLength = this.bufferLength;
		let n = repeatHeader[0];

		if (n < 128) {
			buffer = this.ensureBuffer(bufferLength + n + 1);
			buffer[bufferLength++] = repeatHeader[1];

			if (n > 0) {
				const source = this.str.getBytes(n);
				buffer.set(source, bufferLength);
				bufferLength += n;
			}
		} else {
			n = 257 - n;
			const b = repeatHeader[1];
			buffer = this.ensureBuffer(bufferLength + n + 1);

			for (let i = 0; i < n; i++) {
				buffer[bufferLength++] = b;
			}
		}

		this.bufferLength = bufferLength;
	}
}

export { RunLengthStream };
