import { $getChildren, $uid, $setValue, $getAttributes, $nodeName, $toString } from './xfa_object.js';

class DataHandler {
	constructor(root, data) {
		this.data = data;
		this.dataset = root.datasets || null;
	}

	serialize(storage) {
		const stack = [[-1, this.data[$getChildren]()]];

		while (stack.length > 0) {
			const last = stack.at(-1);
			const [i, children] = last;

			if (i + 1 === children.length) {
				stack.pop();
				continue;
			}

			const child = children[++last[0]];
			const storageEntry = storage.get(child[$uid]);

			if (storageEntry) {
				child[$setValue](storageEntry);
			} else {
				const attributes = child[$getAttributes]();

				for (const value of attributes.values()) {
					const entry = storage.get(value[$uid]);

					if (entry) {
						value[$setValue](entry);
						break;
					}
				}
			}

			const nodes = child[$getChildren]();

			if (nodes.length > 0) {
				stack.push([-1, nodes]);
			}
		}

		const buf = [`<xfa:datasets xmlns:xfa="http://www.xfa.org/schema/xfa-data/1.0/">`];

		if (this.dataset) {
			for (const child of this.dataset[$getChildren]()) {
				if (child[$nodeName] !== "data") {
					child[$toString](buf);
				}
			}
		}

		this.data[$toString](buf);
		buf.push("</xfa:datasets>");
		return buf.join("");
	}
}

export { DataHandler };
