import { XmlObject, $isNsAgnostic, XFAObject, $onChild, $nodeName, $namespaceId, $appendChild } from './xfa_object.js';
import { NamespaceIds, $buildXFAObject } from './namespaces.js';

const DATASETS_NS_ID = NamespaceIds.datasets.id;

class Data extends XmlObject {
	constructor(attributes) {
		super(DATASETS_NS_ID, "data", attributes);
	}

	[$isNsAgnostic]() {
		return true;
	}
}

class Datasets extends XFAObject {
	constructor(attributes) {
		super(DATASETS_NS_ID, "datasets", true);
		this.data = null;
		this.Signature = null;
	}

	[$onChild](child) {
		const name = child[$nodeName];

		if (name === "data" && child[$namespaceId] === DATASETS_NS_ID || name === "Signature" && child[$namespaceId] === NamespaceIds.signature.id) {
			this[name] = child;
		}

		this[$appendChild](child);
	}
}

class DatasetsNamespace {
	static [$buildXFAObject](name, attributes) {
		if (DatasetsNamespace.hasOwnProperty(name)) {
			return DatasetsNamespace[name](attributes);
		}

		return undefined;
	}

	static datasets(attributes) {
		return new Datasets(attributes);
	}

	static data(attributes) {
		return new Data(attributes);
	}
}

export { DatasetsNamespace };
