import { NamespaceIds, $buildXFAObject } from './namespaces.js';
import { XFAObject, XFAObjectArray, $onChildCheck, $nodeName, $namespaceId } from './xfa_object.js';

const XDP_NS_ID = NamespaceIds.xdp.id;

class Xdp extends XFAObject {
	constructor(attributes) {
		super(XDP_NS_ID, "xdp", true);
		this.uuid = attributes.uuid || "";
		this.timeStamp = attributes.timeStamp || "";
		this.config = null;
		this.connectionSet = null;
		this.datasets = null;
		this.localeSet = null;
		this.stylesheet = new XFAObjectArray();
		this.template = null;
	}

	[$onChildCheck](child) {
		const ns = NamespaceIds[child[$nodeName]];
		return ns && child[$namespaceId] === ns.id;
	}
}

class XdpNamespace {
	static [$buildXFAObject](name, attributes) {
		if (XdpNamespace.hasOwnProperty(name)) {
			return XdpNamespace[name](attributes);
		}

		return undefined;
	}

	static xdp(attributes) {
		return new Xdp(attributes);
	}
}

export { XdpNamespace };
