import { MissingPDFException, UnexpectedResponseException } from '../shared/util.js';
import { getFilenameFromContentDispositionHeader } from './content_disposition.js';
import { isPdfFile } from './display_utils.js';

function validateRangeRequestCapabilities({ getResponseHeader, isHttp, rangeChunkSize, disableRange }) {
	const returnValues = {
		allowRangeRequests: false,
		suggestedLength: undefined
	};

	const length = parseInt(getResponseHeader("Content-Length"), 10);

	if (!Number.isInteger(length)) {
		return returnValues;
	}

	returnValues.suggestedLength = length;

	if (length <= 2 * rangeChunkSize) {
		return returnValues;
	}

	if (disableRange || !isHttp) {
		return returnValues;
	}

	if (getResponseHeader("Accept-Ranges") !== "bytes") {
		return returnValues;
	}

	const contentEncoding = getResponseHeader("Content-Encoding") || "identity";

	if (contentEncoding !== "identity") {
		return returnValues;
	}

	returnValues.allowRangeRequests = true;
	return returnValues;
}

function extractFilenameFromHeader(getResponseHeader) {
	const contentDisposition = getResponseHeader("Content-Disposition");

	if (contentDisposition) {
		let filename = getFilenameFromContentDispositionHeader(contentDisposition);

		if (filename.includes("%")) {
			try {
				filename = decodeURIComponent(filename);
			} catch(ex) {
				
			}
		}

		if (isPdfFile(filename)) {
			return filename;
		}
	}

	return null;
}

function createResponseStatusError(status, url) {
	if (status === 404 || status === 0 && url.startsWith("file:")) {
		return new MissingPDFException('Missing PDF "' + url + '".');
	}

	return new UnexpectedResponseException(`Unexpected server response (${status}) while retrieving PDF "${url}".`, status);
}

function validateResponseStatus(status) {
	return status === 200 || status === 206;
}

export { createResponseStatusError, extractFilenameFromHeader, validateRangeRequestCapabilities, validateResponseStatus };
