import require$$0 from 'fs';
import { BaseFilterFactory, BaseCanvasFactory, BaseCMapReaderFactory, BaseStandardFontDataFactory } from './base_factory.js';

const fetchData = function (url) {
	return new Promise((resolve, reject) => {
			const fs = require$$0;

			fs.readFile(url, (error, data) => {
				if (error || !data) {
					reject(new Error(error));
					return;
				}

				resolve(new Uint8Array(data));
			});
		});
};

class NodeFilterFactory extends BaseFilterFactory {
	
}

class NodeCanvasFactory extends BaseCanvasFactory {
	_createCanvas(width, height) {
		const Canvas = require("canvas");
		return Canvas.createCanvas(width, height);
	}
}

class NodeCMapReaderFactory extends BaseCMapReaderFactory {
	_fetchData(url, compressionType) {
		return fetchData(url).then(data => {
			return { cMapData: data, compressionType };
		});
	}
}

class NodeStandardFontDataFactory extends BaseStandardFontDataFactory {
	_fetchData(url) {
		return fetchData(url);
	}
}

export { NodeCMapReaderFactory, NodeCanvasFactory, NodeFilterFactory, NodeStandardFontDataFactory };
